/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Copyright (c) 2010 by disy Informationssysteme GmbH
package net.disy.ogc.wps.v_1_0_0.proxy;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.xml.bind.JAXBElement;

import net.disy.ogc.ows.v_1_1_0.OwsException;
import net.disy.ogc.wps.v_1_0_0.WpsProcess;
import net.disy.ogc.wps.v_1_0_0.util.WpsUtils;
import net.opengis.ows.v_1_1_0.CodeType;
import net.opengis.ows.v_1_1_0.Operation;
import net.opengis.ows.v_1_1_0.RequestMethodType;
import net.opengis.wps.v_1_0_0.ProcessBriefType;
import net.opengis.wps.v_1_0_0.ProcessDescriptionType;
import net.opengis.wps.v_1_0_0.ProcessDescriptions;
import net.opengis.wps.v_1_0_0.WPSCapabilitiesType;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

// NOT_PUBLISHED
public class DefaultProxyWpsProcessFactory implements ProxyWpsProcessFactory {

  private final Log log = LogFactory.getLog(DefaultProxyWpsProcessFactory.class);

  private static final String ID_SEPARATOR = "-"; //$NON-NLS-1$
  private final GenericMarshallingHttpClient httpClient;
  private final PrefixStrategy prefixStrategy;

  public DefaultProxyWpsProcessFactory(
      GenericMarshallingHttpClient httpClient,
      PrefixStrategy prefixStrategy) {
    this.httpClient = httpClient;
    this.prefixStrategy = prefixStrategy;
  }

  @SuppressWarnings("unchecked")
  @Override
  public Collection<WpsProcess> createProxyWpsProcesses(String capabilitiesUrl)
      throws OwsException,
      WPSConnectionException,
      WPSMarshallingException {
    List<WpsProcess> result = new ArrayList<WpsProcess>();
    //FIXME Will not work if process return JAXBElement<Foo>, generics typo info is lost. :(
    JAXBElement<WPSCapabilitiesType> capabilities = ExceptionHandlingUtils.handleExceptionReport(
        httpClient.executeFullyParametrizedGetRequest(capabilitiesUrl),
        JAXBElement.class);
    String describeProcessUrl = null;
    String executeUrl = null;
    for (Operation op : capabilities.getValue().getOperationsMetadata().getOperation()) {
      List<JAXBElement<RequestMethodType>> requestMethods = op
          .getDCP()
          .get(0)
          .getHTTP()
          .getGetOrPost();
      if ("DescribeProcess".equals(op.getName())) { //$NON-NLS-1$
        JAXBElement<RequestMethodType> describeProcessRequestMethod = RequestMethodUtils
            .getGet(requestMethods);
        if (describeProcessRequestMethod != null) {
          String describeProcessBaseUrl = describeProcessRequestMethod.getValue().getHref();
          log.debug("Extracted describe process URL: [" + describeProcessBaseUrl + "]"); //$NON-NLS-1$ //$NON-NLS-2$
          describeProcessUrl = describeProcessBaseUrl
              + (describeProcessBaseUrl.contains("?") ? "" : "?")
              + "service=WPS&request=DescribeProcess&version=1.0.0&identifier="; //$NON-NLS-1$
        }
        else {
          throw new UnsupportedOperationException(
              "Currently only GET DescribeProcess operation is supported"); //$NON-NLS-1$
        }

      }
      else if ("Execute".equals(op.getName())) { //$NON-NLS-1$
        JAXBElement<RequestMethodType> describeProcessRequestMethod = RequestMethodUtils
            .getPost(requestMethods);
        if (describeProcessRequestMethod != null) {
          String executeBaseUrl = describeProcessRequestMethod.getValue().getHref();

          log.debug("Extracted execute URL: [" + executeBaseUrl + "]"); //$NON-NLS-1$ //$NON-NLS-2$
          executeUrl = executeBaseUrl
              + (executeBaseUrl.contains("?") ? "" : "?")
              + "service=WPS&request=Execute&version=1.0.0&identifier="; //$NON-NLS-1$
        }
        else {
          throw new UnsupportedOperationException("Only POST is supported for Execute"); //$NON-NLS-1$
        }
      }
    } //for
    String proxyPrefix = prefixStrategy.createPrefixFromURL(capabilitiesUrl);
    log.debug("Created prefix [" //$NON-NLS-1$
        + proxyPrefix
        + "] for the capabilities URL [" //$NON-NLS-1$
        + capabilitiesUrl
        + "]"); //$NON-NLS-1$
    for (ProcessBriefType process : capabilities.getValue().getProcessOfferings().getProcess()) {
      CodeType remoteId = process.getIdentifier();
      String describeProcessUrlWithId = describeProcessUrl + remoteId.getValue();
      String executeUrlWithId = executeUrl + remoteId.getValue();
      ProcessDescriptions processes = ExceptionHandlingUtils.handleExceptionReport(
          httpClient.executeFullyParametrizedGetRequest(describeProcessUrlWithId),
          ProcessDescriptions.class);
      ProcessDescriptionType remoteDescription = processes.getProcessDescription().get(0);
      CodeType proxyId = WpsUtils.createCodeType(proxyPrefix + ID_SEPARATOR + remoteId.getValue());
      log.debug("Created proxy id [" + proxyId.getValue() + "]"); //$NON-NLS-1$ //$NON-NLS-2$
      WpsProcess proxyProcess = new ProxyProcess(
          proxyId,
          executeUrlWithId,
          remoteDescription,
          httpClient);
      log.debug("Created proxy process: " + proxyProcess); //$NON-NLS-1$
      result.add(proxyProcess);
    }

    return result;
  }
}
