/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Copyright (c) 2010 by disy Informationssysteme GmbH
package net.disy.ogc.wps.v_1_0_0.proxy;

import net.disy.ogc.ows.v_1_1_0.NoApplicableCodeException;
import net.disy.ogc.ows.v_1_1_0.OwsException;
import net.opengis.ows.v_1_1_0.ExceptionReport;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

// NOT_PUBLISHED
public class ExceptionHandlingUtils {

  private static Log log = LogFactory.getLog(ExceptionHandlingUtils.class);

  @SuppressWarnings("unchecked")
  public static <T> T handleExceptionReport(Object responseObject, Class<T> expectedType)
      throws OwsException {

    if (expectedType.isAssignableFrom(responseObject.getClass())) {
      return (T) responseObject;
    }
    else if (responseObject instanceof ExceptionReport) {
      ExceptionReport exceptionReport = (ExceptionReport) responseObject;
      log.warn("Remote process returned an exception report: " + exceptionReport); //$NON-NLS-1$
      OwsException wrapperException = new OwsException(exceptionReport);
      throw wrapperException;
    }
    else {
      log.error("Seeing unexpected response class: " + responseObject.getClass()); //$NON-NLS-1$
      throw new NoApplicableCodeException("Unexpected response class: " + responseObject.getClass()); //$NON-NLS-1$
    }

  }

}
