/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Copyright (c) 2010 by disy Informationssysteme GmbH
package net.disy.ogc.wps.v_1_0_0.proxy;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

// NOT_PUBLISHED
public class HostPrefixStrategy implements PrefixStrategy {

  private static final String LOCALHOST = "localhost"; //$NON-NLS-1$
  private static final String FILE_PROTOCOL = "file"; //$NON-NLS-1$
  private static final String INDEX_SEPARATOR = "-"; //$NON-NLS-1$
  private final Map<String, Integer> generatedPrefixes;

  public HostPrefixStrategy() {
    generatedPrefixes = new HashMap<String, Integer>();
  }

  @Override
  public synchronized String createPrefixFromURL(String url) throws WPSConnectionException {
    if (url == null) {
      throw new WPSConnectionException("url should not be null"); //$NON-NLS-1$
    }
    URL givenUrl;
    try {
      givenUrl = new URL(url);
    }
    catch (MalformedURLException e) {
      throw new WPSConnectionException("The provided url [" + url + "] is not a valid URL.");
    }
    String host = extractHostName(givenUrl);
    if (host != null && host.length() > 0) {
      if (generatedPrefixes.containsKey(host)) {
        int lastIndex = generatedPrefixes.get(host);
        int newIndex = lastIndex + 1;
        generatedPrefixes.put(host, newIndex);
        host = host + INDEX_SEPARATOR + newIndex;
      }
      else {
        generatedPrefixes.put(host, 1);
      }
      return host;
    }
    throw new WPSConnectionException(
        "No host could been identified in the provided URL [" + url + "]"); //$NON-NLS-1$ //$NON-NLS-2$
  }

  private String extractHostName(URL url) {
    String host = null;
    if (FILE_PROTOCOL.equals(url.getProtocol())) {
      host = LOCALHOST;
    }
    else {
      host = url.getHost();
    }
    return host;
  }
}
