/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Copyright (c) 2010 by disy Informationssysteme GmbH
package net.disy.ogc.wps.v_1_0_0.proxy;

import net.disy.ogc.ows.v_1_1_0.NoApplicableCodeException;
import net.disy.ogc.ows.v_1_1_0.OwsException;
import net.disy.ogc.wps.v_1_0_0.WpsProcess;
import net.opengis.ows.v_1_1_0.CodeType;
import net.opengis.wps.v_1_0_0.DataInputsType;
import net.opengis.wps.v_1_0_0.Execute;
import net.opengis.wps.v_1_0_0.ExecuteResponse;
import net.opengis.wps.v_1_0_0.ProcessDescriptionType;
import net.opengis.wps.v_1_0_0.ResponseFormType;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

// NOT_PUBLISHED
public class ProxyProcess implements WpsProcess {

  private final Log log = LogFactory.getLog(getClass());

  private final CodeType remoteIdentifier;
  private final ProcessDescriptionType ownDescription;
  private final GenericMarshallingHttpClient httpClient;
  private final String executeUrl;

  public ProxyProcess(
      CodeType proxyId,
      String executeUrl,
      ProcessDescriptionType remoteDescription,
      GenericMarshallingHttpClient httpClient) {
    this.httpClient = httpClient;
    remoteIdentifier = remoteDescription.getIdentifier();
    ownDescription = (ProcessDescriptionType) remoteDescription.copyTo(remoteDescription
        .createNewInstance());
    ownDescription.setIdentifier(proxyId);
    this.executeUrl = executeUrl;
  }

  @Override
  public ExecuteResponse execute(
      DataInputsType dataInputs,
      ResponseFormType responseForm,
      String language) throws OwsException {
    final Execute execute = createProxyExecution(dataInputs, responseForm, language);
    return executeInternal(execute);
  }

  /**
   * Public only for easier testing purposes
   */
  public Execute createProxyExecution(
      DataInputsType dataInputs,
      ResponseFormType responseForm,
      String language) {
    final Execute execute = new Execute();
    execute.setService("WPS"); //$NON-NLS-1$
    execute.setVersion("1.0.0"); //$NON-NLS-1$
    execute.setLanguage(language);
    execute.setIdentifier(remoteIdentifier);
    execute.setDataInputs(dataInputs);
    execute.setResponseForm(responseForm);
    execute.setLanguage(language);
    return execute;
  }

  private ExecuteResponse executeInternal(Execute proxyExecute) throws OwsException {
    Object responseObject;
    ExecuteResponse response = null;
    try {
      responseObject = httpClient.executePostRequest(executeUrl, proxyExecute);
    }
    catch (Exception e) {
      if (e instanceof OwsException) {
        throw (OwsException) e;
      }
      log.error("Unexpected internal exception happened", e); //$NON-NLS-1$
      throw new NoApplicableCodeException("Internal error", e); //$NON-NLS-1$
    }
    response = ExceptionHandlingUtils.handleExceptionReport(responseObject, ExecuteResponse.class);
    if (response != null) {
      response.getProcess().setIdentifier(ownDescription.getIdentifier());
    }
    return response;
  }

  @Override
  public ProcessDescriptionType getProcessDescription() {
    return ownDescription;
  }

}
