/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.proxy;

import java.util.Collection;
import java.util.List;

import net.disy.ogc.wps.v_1_0_0.WpsProcess;
import net.disy.ogc.wps.v_1_0_0.WpsProcessRegistry;

public class ProxyProcessAdder {

  private final ProxyWpsProcessFactory proxyFactory;
  private final WpsProcessRegistry processRegistry;

  public ProxyProcessAdder(WpsProcessRegistry processRegistry, ProxyWpsProcessFactory proxyFactory) {
    this.processRegistry = processRegistry;
    this.proxyFactory = proxyFactory;
  }

  public void addProxyProcesses(final List<String> capabilitiesUrls)
      throws IllegalArgumentException {
    for (String capabilitiesUrl : capabilitiesUrls) {
      addProxyProcess(proxyFactory, capabilitiesUrl);
    }
  }

  private void addProxyProcess(final ProxyWpsProcessFactory proxyFactory, String capabilitiesUrl)
      throws IllegalArgumentException {
    try {
      if (capabilitiesUrl != null) {
        Collection<WpsProcess> proxyProcesses = proxyFactory
            .createProxyWpsProcesses(capabilitiesUrl.trim());
        for (WpsProcess proxyProcess : proxyProcesses) {
          processRegistry.registerProcess(proxyProcess);
        }
      }
    }
    catch (Exception e) {
      throw new IllegalArgumentException(
          "Could not create proxy processes for the capabilities URL [" //$NON-NLS-1$
              + capabilitiesUrl
              + "].", e); //$NON-NLS-1$
    }
  }
}
