/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.proxy.servlet;

import java.util.Arrays;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;

import net.disy.ogc.wps.v_1_0_0.proxy.DefaultProxyWpsProcessFactory;
import net.disy.ogc.wps.v_1_0_0.proxy.GenericMarshallingHttpClient;
import net.disy.ogc.wps.v_1_0_0.proxy.HostPrefixStrategy;
import net.disy.ogc.wps.v_1_0_0.proxy.ProxyProcessAdder;
import net.disy.ogc.wps.v_1_0_0.proxy.ProxyWpsProcessFactory;
import net.disy.ogc.wps.v_1_0_0.util.WpsOperationsUtilities;

import org.apache.commons.lang.StringUtils;

public class WpsProxyConfigurationServlet extends HttpServlet {

  private static final long serialVersionUID = 1L;
  public static final String CAPABILITIES_INIT_PARAMETER_NAME = "capabilities"; //$NON-NLS-1$

  @Override
  public void init(ServletConfig config) throws ServletException {
    super.init(config);

    final ProxyWpsProcessFactory proxyFactory = new DefaultProxyWpsProcessFactory(
        new GenericMarshallingHttpClient(WpsOperationsUtilities.getWpsProcessContext(
            config.getServletContext()).getJAXBContext()),
        new HostPrefixStrategy());

    String capabilitiesUrlsString = config.getInitParameter(CAPABILITIES_INIT_PARAMETER_NAME);
    if (capabilitiesUrlsString != null) {
      final String[] capabilitiesUrls = StringUtils.split(capabilitiesUrlsString, ',');
      try {
        new ProxyProcessAdder(WpsOperationsUtilities.getWpsProcessRegistry(config
            .getServletContext()), proxyFactory).addProxyProcesses(Arrays.asList(capabilitiesUrls));
      }
      catch (Exception e) {
        throw new ServletException("Could not create proxy processes.", e); //$NON-NLS-1$
      }

    } // if != null
  }

}
